/***************************************
	Auteur : Pierre Aubert
	Mail : pierre.aubert@lapp.in2p3.fr
	Licence : CeCILL-C
****************************************/
#include <iostream>
#include <ranges>
#include <vector>

#include <algorithm>

//Some doc at : https://en.cppreference.com/w/cpp/header/execution
#include <execution>


///Defines a vector of data
typedef std::vector<float> VecData;

///Do the Hadamard product
/**	@param[out] vecRes : vector of results of tabX*tabY
 * 	@param vecX : input vector
 * 	@param vecY : input vector
 * 	@param vecZ : input vector
*/
void multi_hadamard_product(VecData & vecRes, const VecData & vecX, const VecData & vecY, const VecData & vecZ){
	auto vXY = std::views::transform(vecX,
			[](float x){
				return x*x;
			}
	);
// 	auto vXY = std::views::transform(std::views::zip(vecX, vecY),
// 			[](auto tuple){
// 				auto & [x, y] = tuple;
// 				return x*y;
// 			}
// 	);
	std::transform(std::execution::par_unseq, std::begin(vXY), std::end(vXY), std::begin(vecZ), std::begin(vecRes),
			[](float xy, float z){
				return xy *z;
			}
		);
}


int main(int argc, char** argv){
	size_t nbElement(100000000);
	std::vector<float> tabX, tabY, tabZ, tabRes;
	tabRes.resize(nbElement);
	for(size_t i(0lu); i < nbElement; ++i){
		tabX.push_back(i*19lu%11);
		tabY.push_back(i*27lu%19);
		tabZ.push_back(i*31lu%17);
	}
	multi_hadamard_product(tabRes, tabX, tabY, tabZ);
	
// 	std::transform(std::execution::par_unseq, std::begin(tabX), std::end(tabX), std::begin(tabY), std::begin(tabRes),
// 			[](float xi, float yi){ return xi * yi; });
	std::cout << "x = " << tabX.front() << ", y = " << tabY.front() << ", res = " << tabRes.front() << std::endl;
	return 0;
}

